@@ -94,7 +94,7 @@ lockdownd_error_t lockdownd_goodbye(lockdownd_client_t client);
 
 /* Helper */
 void lockdownd_client_set_label(lockdownd_client_t client, const char *label);
-lockdownd_error_t lockdownd_get_device_uuid(lockdownd_client_t control, char **uuid);
+lockdownd_error_t lockdownd_get_device_udid(lockdownd_client_t control, char **udid);
 lockdownd_error_t lockdownd_get_device_name(lockdownd_client_t client, char **device_name);
 lockdownd_error_t lockdownd_get_sync_data_classes(lockdownd_client_t client, char ***classes, int *count);
 lockdownd_error_t lockdownd_data_classes_free(char **classes);
