@@ -19,6 +19,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -27,6 +30,7 @@
 #include "afc.h"
 #include "idevice.h"
 #include "debug.h"
+#include "endianness.h"
 
 /** The maximum size an AFC data packet can be */
 static const int MAXIMUM_PACKET_SIZE = (2 << 15);
@@ -39,7 +43,11 @@ static const int MAXIMUM_PACKET_SIZE = (2 << 15);
 static void afc_lock(afc_client_t client)
 {
 	debug_info("Locked");
-	g_mutex_lock(client->mutex);
+#ifdef WIN32
+	EnterCriticalSection(&client->mutex);
+#else
+	pthread_mutex_lock(&client->mutex);
+#endif
 }
 
 /**
@@ -50,7 +58,11 @@ static void afc_lock(afc_client_t client)
 static void afc_unlock(afc_client_t client)
 {
 	debug_info("Unlocked");
-	g_mutex_unlock(client->mutex);
+#ifdef WIN32
+	LeaveCriticalSection(&client->mutex);
+#else
+	pthread_mutex_unlock(&client->mutex);
+#endif
 }
 
 /**
@@ -69,10 +81,6 @@ static void afc_unlock(afc_client_t client)
 
 afc_error_t afc_client_new_from_connection(idevice_connection_t connection, afc_client_t *client)
 {
-	/* makes sure thread environment is available */
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-
 	if (!connection)
 		return AFC_E_INVALID_ARG;
 
@@ -93,7 +101,11 @@ afc_error_t afc_client_new_from_connection(idevice_connection_t connection, afc_
 	memcpy(client_loc->afc_packet->magic, AFC_MAGIC, AFC_MAGIC_LEN);
 	client_loc->file_handle = 0;
 	client_loc->lock = 0;
-	client_loc->mutex = g_mutex_new();
+#ifdef WIN32
+	InitializeCriticalSection(&client_loc->mutex);
+#else
+	pthread_mutex_init(&client_loc->mutex, NULL);
+#endif
 
 	*client = client_loc;
 	return AFC_E_SUCCESS;
@@ -117,10 +129,6 @@ afc_error_t afc_client_new_from_connection(idevice_connection_t connection, afc_
  */
 afc_error_t afc_client_new(idevice_t device, uint16_t port, afc_client_t * client)
 {
-	/* makes sure thread environment is available */
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-
 	if (!device || port==0)
 		return AFC_E_INVALID_ARG;
 
@@ -155,9 +163,11 @@ afc_error_t afc_client_free(afc_client_t client)
 		client->connection = NULL;
 	}
 	free(client->afc_packet);
-	if (client->mutex) {
-		g_mutex_free(client->mutex);
-	}
+#ifdef WIN32
+	DeleteCriticalSection(&client->mutex);
+#else
+	pthread_mutex_destroy(&client->mutex);
+#endif
 	free(client);
 	return AFC_E_SUCCESS;
 }
@@ -374,7 +384,7 @@ static afc_error_t afc_receive_data(afc_client_t client, char **dump_here, uint3
 	}
 
 	if (current_count >= sizeof(uint64_t)) {
-		param1 = GUINT64_FROM_LE(*(uint64_t*)(*dump_here));
+		param1 = le64toh(*(uint64_t*)(*dump_here));
 	}
 
 	debug_info("packet data size = %i", current_count);
@@ -409,7 +419,9 @@ static afc_error_t afc_receive_data(afc_client_t client, char **dump_here, uint3
 		*bytes_recv = 0;
 
 		debug_info("WARNING: Unknown operation code received 0x%llx param1=%lld", header.operation, param1);
+#ifndef WIN32
 		fprintf(stderr, "%s: WARNING: Unknown operation code received 0x%llx param1=%lld", __func__, (long long)header.operation, (long long)param1);
+#endif
 
 		return AFC_E_OP_NOT_SUPPORTED;
 	}
@@ -587,8 +599,10 @@ afc_error_t afc_get_device_info_key(afc_client_t client, const char *key, char *
 			break;
 		}
 	}
-
-	g_strfreev(kvps);
+	for (ptr = kvps; *ptr; ptr++) {
+		free(*ptr);
+	}
+	free(kvps);
 
 	return ret;
 }
@@ -774,7 +788,7 @@ idevice_error_t
 afc_file_open(afc_client_t client, const char *filename,
 					 afc_file_mode_t file_mode, uint64_t *handle)
 {
-	uint64_t file_mode_loc = GUINT64_TO_LE(file_mode);
+	uint64_t file_mode_loc = htole64(file_mode);
 	uint32_t bytes = 0;
 	char *data = (char *) malloc(sizeof(char) * (8 + strlen(filename) + 1));
 	afc_error_t ret = AFC_E_UNKNOWN_ERROR;
@@ -852,7 +866,7 @@ afc_file_read(afc_client_t client, uint64_t handle, char *data, uint32_t length,
 		/* Send the read command */
 		AFCFilePacket *packet = (AFCFilePacket *) malloc(sizeof(AFCFilePacket));
 		packet->filehandle = handle;
-		packet->size = GUINT64_TO_LE(((length - current_count) < MAXIMUM_READ_SIZE) ? (length - current_count) : MAXIMUM_READ_SIZE);
+		packet->size = htole64(((length - current_count) < MAXIMUM_READ_SIZE) ? (length - current_count) : MAXIMUM_READ_SIZE);
 		client->afc_packet->operation = AFC_OP_READ;
 		client->afc_packet->entire_length = client->afc_packet->this_length = 0;
 		ret = afc_dispatch_packet(client, (char *) packet, sizeof(AFCFilePacket), &bytes_loc);
@@ -1045,7 +1059,7 @@ afc_error_t afc_file_lock(afc_client_t client, uint64_t handle, afc_lock_op_t op
 {
 	char *buffer = malloc(16);
 	uint32_t bytes = 0;
-	uint64_t op = GUINT64_TO_LE(operation);
+	uint64_t op = htole64(operation);
 	afc_error_t ret = AFC_E_UNKNOWN_ERROR;
 
 	if (!client || (handle == 0))
@@ -1094,8 +1108,8 @@ afc_error_t afc_file_lock(afc_client_t client, uint64_t handle, afc_lock_op_t op
 afc_error_t afc_file_seek(afc_client_t client, uint64_t handle, int64_t offset, int whence)
 {
 	char *buffer = (char *) malloc(sizeof(char) * 24);
-	int64_t offset_loc = (int64_t)GUINT64_TO_LE(offset);
-	uint64_t whence_loc = GUINT64_TO_LE(whence);
+	int64_t offset_loc = (int64_t)htole64(offset);
+	uint64_t whence_loc = htole64(whence);
 	uint32_t bytes = 0;
 	afc_error_t ret = AFC_E_UNKNOWN_ERROR;
 
@@ -1166,7 +1180,7 @@ afc_error_t afc_file_tell(afc_client_t client, uint64_t handle, uint64_t *positi
 	if (bytes > 0 && buffer) {
 		/* Get the position */
 		memcpy(position, buffer, sizeof(uint64_t));
-		*position = GUINT64_FROM_LE(*position);
+		*position = le64toh(*position);
 	}
 	if (buffer)
 		free(buffer);
@@ -1192,7 +1206,7 @@ afc_error_t afc_file_truncate(afc_client_t client, uint64_t handle, uint64_t new
 {
 	char *buffer = (char *) malloc(sizeof(char) * 16);
 	uint32_t bytes = 0;
-	uint64_t newsize_loc = GUINT64_TO_LE(newsize);
+	uint64_t newsize_loc = htole64(newsize);
 	afc_error_t ret = AFC_E_UNKNOWN_ERROR;
 
 	if (!client || (handle == 0))
@@ -1237,7 +1251,7 @@ afc_error_t afc_truncate(afc_client_t client, const char *path, uint64_t newsize
 	char *response = NULL;
 	char *send = (char *) malloc(sizeof(char) * (strlen(path) + 1 + 8));
 	uint32_t bytes = 0;
-	uint64_t size_requested = GUINT64_TO_LE(newsize);
+	uint64_t size_requested = htole64(newsize);
 	afc_error_t ret = AFC_E_UNKNOWN_ERROR;
 
 	if (!client || !path || !client->afc_packet || !client->connection)
@@ -1281,7 +1295,7 @@ afc_error_t afc_make_link(afc_client_t client, afc_link_type_t linktype, const c
 	char *response = NULL;
 	char *send = (char *) malloc(sizeof(char) * (strlen(target)+1 + strlen(linkname)+1 + 8));
 	uint32_t bytes = 0;
-	uint64_t type = GUINT64_TO_LE(linktype);
+	uint64_t type = htole64(linktype);
 	afc_error_t ret = AFC_E_UNKNOWN_ERROR;
 
 	if (!client || !target || !linkname || !client->afc_packet || !client->connection)
@@ -1329,7 +1343,7 @@ afc_error_t afc_set_file_time(afc_client_t client, const char *path, uint64_t mt
 	char *response = NULL;
 	char *send = (char *) malloc(sizeof(char) * (strlen(path) + 1 + 8));
 	uint32_t bytes = 0;
-	uint64_t mtime_loc = GUINT64_TO_LE(mtime);
+	uint64_t mtime_loc = htole64(mtime);
 	afc_error_t ret = AFC_E_UNKNOWN_ERROR;
 
 	if (!client || !path || !client->afc_packet || !client->connection)
