@@ -19,10 +19,15 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
  */
 
-#include <glib.h>
 #include <stdint.h>
+#ifdef WIN32
+#include <windows.h>
+#else
+#include <pthread.h>
+#endif
 
 #include "libimobiledevice/afc.h"
+#include "endianness.h"
 
 #define AFC_MAGIC "CFA6LPAA"
 #define AFC_MAGIC_LEN (8)
@@ -33,16 +38,16 @@ typedef struct {
 } AFCPacket;
 
 #define AFCPacket_to_LE(x) \
- 	(x)->entire_length = GUINT64_TO_LE((x)->entire_length); \
-	(x)->this_length   = GUINT64_TO_LE((x)->this_length); \
-	(x)->packet_num    = GUINT64_TO_LE((x)->packet_num); \
-	(x)->operation     = GUINT64_TO_LE((x)->operation);
+ 	(x)->entire_length = htole64((x)->entire_length); \
+	(x)->this_length   = htole64((x)->this_length); \
+	(x)->packet_num    = htole64((x)->packet_num); \
+	(x)->operation     = htole64((x)->operation);
 
 #define AFCPacket_from_LE(x) \
-	(x)->entire_length = GUINT64_FROM_LE((x)->entire_length); \
-	(x)->this_length   = GUINT64_FROM_LE((x)->this_length); \
-	(x)->packet_num    = GUINT64_FROM_LE((x)->packet_num); \
-	(x)->operation     = GUINT64_FROM_LE((x)->operation);
+	(x)->entire_length = le64toh((x)->entire_length); \
+	(x)->this_length   = le64toh((x)->this_length); \
+	(x)->packet_num    = le64toh((x)->packet_num); \
+	(x)->operation     = le64toh((x)->operation);
 
 typedef struct {
 	uint64_t filehandle, size;
@@ -53,7 +58,11 @@ struct afc_client_private {
 	AFCPacket *afc_packet;
 	int file_handle;
 	int lock;
-	GMutex *mutex;
+#ifdef WIN32
+	CRITICAL_SECTION mutex;
+#else
+	pthread_mutex_t mutex;
+#endif
 	int own_connection;
 };
 
