@@ -24,15 +24,6 @@
 #include "libimobiledevice/file_relay.h"
 #include "property_list_service.h"
 
-/* Error Codes */
-#define FILE_RELAY_E_SUCCESS                0
-#define FILE_RELAY_E_INVALID_ARG           -1
-#define FILE_RELAY_E_PLIST_ERROR           -2
-#define FILE_RELAY_E_MUX_ERROR             -3
-
-#define FILE_RELAY_E_UNKNOWN_ERROR       -256
-
-
 struct file_relay_client_private {
 	property_list_service_client_t parent;
 };
