@@ -21,15 +21,24 @@
 #ifndef IINSTALLATION_PROXY_H
 #define IINSTALLATION_PROXY_H
 
-#include <glib.h>
+#ifdef WIN32
+#include <windows.h>
+#else
+#include <pthread.h>
+#endif
 
 #include "libimobiledevice/installation_proxy.h"
 #include "property_list_service.h"
 
 struct instproxy_client_private {
 	property_list_service_client_t parent;
-	GMutex *mutex;
-	GThread *status_updater;
+#ifdef WIN32
+	CRITICAL_SECTION mutex;
+	HANDLE status_updater;
+#else
+	pthread_mutex_t mutex;
+	pthread_t status_updater;
+#endif
 };
 
 #endif
