@@ -28,6 +28,10 @@
 #include "property_list_service.h"
 #include "debug.h"
 
+#ifdef WIN32
+#define sleep(x) Sleep(x*1000)
+#endif
+
 struct np_thread {
 	np_client_t client;
 	np_notify_cb_t cbfunc;
@@ -42,7 +46,11 @@ struct np_thread {
 static void np_lock(np_client_t client)
 {
 	debug_info("NP: Locked");
-	g_mutex_lock(client->mutex);
+#ifdef WIN32
+	EnterCriticalSection(&client->mutex);
+#else
+	pthread_mutex_lock(&client->mutex);
+#endif
 }
 
 /**
@@ -53,7 +61,11 @@ static void np_lock(np_client_t client)
 static void np_unlock(np_client_t client)
 {
 	debug_info("NP: Unlocked");
-	g_mutex_unlock(client->mutex);
+#ifdef WIN32
+	LeaveCriticalSection(&client->mutex);
+#else
+	pthread_mutex_unlock(&client->mutex);
+#endif
 }
 
 /**
@@ -96,10 +108,6 @@ static np_error_t np_error(property_list_service_error_t err)
  */
 np_error_t np_client_new(idevice_t device, uint16_t port, np_client_t *client)
 {
-	/* makes sure thread environment is available */
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-
 	if (!device)
 		return NP_E_INVALID_ARG;
 
@@ -111,9 +119,13 @@ np_error_t np_client_new(idevice_t device, uint16_t port, np_client_t *client)
 	np_client_t client_loc = (np_client_t) malloc(sizeof(struct np_client_private));
 	client_loc->parent = plistclient;
 
-	client_loc->mutex = g_mutex_new();
-
+#ifdef WIN32
+	InitializeCriticalSection(&client_loc->mutex);
 	client_loc->notifier = NULL;
+#else
+	pthread_mutex_init(&client_loc->mutex, NULL);
+	client_loc->notifier = (pthread_t)NULL;
+#endif
 
 	*client = client_loc;
 	return NP_E_SUCCESS;
@@ -136,11 +148,17 @@ np_error_t np_client_free(np_client_t client)
 	client->parent = NULL;
 	if (client->notifier) {
 		debug_info("joining np callback");
-		g_thread_join(client->notifier);
-	}
-	if (client->mutex) {
-		g_mutex_free(client->mutex);
+#ifdef WIN32
+		WaitForSingleObject(client->notifier, INFINITE);
+#else
+		pthread_join(client->notifier, NULL);
+#endif
 	}
+#ifdef WIN32
+	DeleteCriticalSection(&client->mutex);
+#else
+	pthread_mutex_destroy(&client->mutex);
+#endif
 	free(client);
 
 	return NP_E_SUCCESS;
@@ -194,7 +212,9 @@ np_error_t np_post_notification(np_client_t client, const char *notification)
 		} else {
 			debug_plist(dict);
 		}
-		g_free(cmd_value);
+		if (cmd_value) {
+			free(cmd_value);
+		}
 #endif
 		plist_free(dict);
 	}
@@ -342,7 +362,7 @@ static int np_get_notification(np_client_t client, char **notification)
 /**
  * Internally used thread function.
  */
-gpointer np_notifier( gpointer arg )
+void* np_notifier( void* arg )
 {
 	char *notification = NULL;
 	struct np_thread *npt = (struct np_thread*)arg;
@@ -397,8 +417,13 @@ np_error_t np_set_notify_callback( np_client_t client, np_notify_cb_t notify_cb,
 		debug_info("callback already set, removing\n");
 		property_list_service_client_t parent = client->parent;
 		client->parent = NULL;
-		g_thread_join(client->notifier);
+#ifdef WIN32
+		WaitForSingleObject(client->notifier, INFINITE);
 		client->notifier = NULL;
+#else
+		pthread_join(client->notifier, NULL);
+		client->notifier = (pthread_t)NULL;
+#endif
 		client->parent = parent;
 	}
 
@@ -409,10 +434,18 @@ np_error_t np_set_notify_callback( np_client_t client, np_notify_cb_t notify_cb,
 			npt->cbfunc = notify_cb;
 			npt->user_data = user_data;
 
-			client->notifier = g_thread_create(np_notifier, npt, TRUE, NULL);
-			if (client->notifier) {
+#ifdef WIN32
+			client->notifier = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)np_notifier, npt, 0, NULL);
+			if (client->notifier != INVALID_HANDLE_VALUE) {
+				res = NP_E_SUCCESS;
+			} else {
+				client->notifier = NULL;
+			}
+#else
+			if (pthread_create(&client->notifier, NULL, np_notifier, npt) == 0) {
 				res = NP_E_SUCCESS;
 			}
+#endif
 		}
 	} else {
 		debug_info("no callback set");
