@@ -21,17 +21,26 @@
 #ifndef INOTIFICATION_PROXY_H
 #define INOTIFICATION_PROXY_H
 
-#include <glib.h>
+#ifdef WIN32
+#include <windows.h>
+#else
+#include <pthread.h>
+#endif
 
 #include "libimobiledevice/notification_proxy.h"
 #include "property_list_service.h"
 
 struct np_client_private {
 	property_list_service_client_t parent;
-	GMutex *mutex;
-	GThread *notifier;
+#ifdef WIN32
+	CRITICAL_SECTION mutex;
+	HANDLE notifier;
+#else
+	pthread_mutex_t mutex;
+	pthread_t notifier;
+#endif
 };
 
-gpointer np_notifier(gpointer arg);
+void* np_notifier(void* arg);
 
 #endif
