@@ -18,13 +18,16 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
-#include <glib.h>
 
 #include "property_list_service.h"
 #include "idevice.h"
 #include "debug.h"
+#include "endianness.h"
 
 /**
  * Convert an idevice_error_t value to an property_list_service_error_t value.
@@ -138,7 +141,7 @@ static property_list_service_error_t internal_plist_send(property_list_service_c
 		return PROPERTY_LIST_SERVICE_E_PLIST_ERROR;
 	}
 
-	nlen = GUINT32_TO_BE(length);
+	nlen = htobe32(length);
 	debug_info("sending %d bytes", length);
 	idevice_connection_send(client->connection, (const char*)&nlen, sizeof(nlen), (uint32_t*)&bytes);
 	if (bytes == sizeof(nlen)) {
@@ -226,7 +229,7 @@ static property_list_service_error_t internal_plist_receive_timeout(property_lis
 		debug_info("initial read failed!");
 		return PROPERTY_LIST_SERVICE_E_MUX_ERROR;
 	} else {
-		pktlen = GUINT32_FROM_BE(pktlen);
+		pktlen = be32toh(pktlen);
 		if (pktlen < (1 << 24)) { /* prevent huge buffers */
 			uint32_t curlen = 0;
 			char *content = NULL;
@@ -245,9 +248,9 @@ static property_list_service_error_t internal_plist_receive_timeout(property_lis
 			if (!memcmp(content, "bplist00", 8)) {
 				plist_from_bin(content, pktlen, plist);
 			} else {
-				/* iOS 4.3 hack: plist data might contain invalid null characters, thus we convert those to spaces */
+				/* iOS 4.3+ hack: plist data might contain invalid characters, thus we convert those to spaces */
 				for (bytes = 0; bytes < pktlen-1; bytes++) {
-					if (content[bytes] == 0x0)
+					if ((content[bytes] < 0x20) && (content[bytes] != 0x09) && (content[bytes] != 0x0a) && (content[bytes] != 0x0d))
 						content[bytes] = 0x20;
 				}
 				plist_from_xml(content, pktlen, plist);
