@@ -25,10 +25,11 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <signal.h>
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <gcrypt.h>
 #include <unistd.h>
+#include <dirent.h>
+#include <libgen.h>
+#include <ctype.h>
+#include <time.h>
 
 #include <libimobiledevice/libimobiledevice.h>
 #include <libimobiledevice/lockdown.h>
@@ -36,12 +37,18 @@
 #include <libimobiledevice/notification_proxy.h>
 #include <libimobiledevice/afc.h>
 
+#include <endianness.h>
+
 #define MOBILEBACKUP2_SERVICE_NAME "com.apple.mobilebackup2"
 #define NP_SERVICE_NAME "com.apple.mobile.notification_proxy"
 
 #define LOCK_ATTEMPTS 50
 #define LOCK_WAIT 200000
 
+#ifdef WIN32
+#define sleep(x) Sleep(x*1000)
+#endif
+
 #define CODE_SUCCESS 0x00
 #define CODE_ERROR_LOCAL 0x06
 #define CODE_ERROR_REMOTE 0x0b
@@ -72,10 +79,10 @@ enum plist_format_t {
 };
 
 enum cmd_flags {
-	CMD_FLAG_RESTORE_SYSTEM_FILES       = 0,
-	CMD_FLAG_RESTORE_REBOOT             = (1 << 1),
-	CMD_FLAG_RESTORE_COPY_BACKUP        = (1 << 2),
-	CMD_FLAG_RESTORE_SETTINGS           = (1 << 3)
+	CMD_FLAG_RESTORE_SYSTEM_FILES       = (1 << 1),
+	CMD_FLAG_RESTORE_REBOOT             = (1 << 2),
+	CMD_FLAG_RESTORE_COPY_BACKUP        = (1 << 3),
+	CMD_FLAG_RESTORE_SETTINGS           = (1 << 4)
 };
 
 static void notify_cb(const char *notification, void *userdata)
@@ -153,14 +160,101 @@ static void mobilebackup_afc_get_file_contents(const char *filename, char **data
 	afc_file_close(afc, f);
 }
 
+static char *str_toupper(char* str)
+{
+	char *res = strdup(str);
+	unsigned int i;
+	for (i = 0; i < strlen(res); i++) {
+		res[i] = toupper(res[i]);
+	}
+	return res;
+}
+
+static int __mkdir(const char* path, int mode)
+{
+#ifdef WIN32
+	return mkdir(path);
+#else
+	return mkdir(path, mode);
+#endif
+}
+
+static int mkdir_with_parents(const char *dir, int mode)
+{
+	if (!dir) return -1;
+	if (__mkdir(dir, mode) == 0) {
+		return 0;
+	} else {
+		if (errno == EEXIST) return 0;	
+	}
+	int res;
+	char *parent = strdup(dir);
+	parent = dirname(parent);
+	if (parent) {
+		res = mkdir_with_parents(parent, mode);
+	} else {
+		res = -1;	
+	}
+	free(parent);
+	if (res == 0) {
+		mkdir_with_parents(dir, mode);
+	}
+	return res;
+}
+
+static char* build_path(const char* elem, ...)
+{
+	if (!elem) return NULL;
+	va_list args;
+	int len = strlen(elem)+1;
+	va_start(args, elem);
+	char *arg = va_arg(args, char*);
+	while (arg) {
+		len += strlen(arg)+1;
+		arg = va_arg(args, char*);
+	}
+	va_end(args);
+
+	char* out = (char*)malloc(len);
+	strcpy(out, elem);
+
+	va_start(args, elem);
+	arg = va_arg(args, char*);
+	while (arg) {
+		strcat(out, "/");
+		strcat(out, arg);
+		arg = va_arg(args, char*);
+	}
+	va_end(args);
+	return out;
+}
+
+static char* format_size_for_display(uint64_t size)
+{
+	char buf[32];
+	double sz;
+	if (size >= 1000000000LL) {
+		sz = ((double)size / 1000000000.0f);
+		sprintf(buf, "%0.1f GB", sz);
+	} else if (size >= 1000000LL) {
+		sz = ((double)size / 1000000.0f);
+		sprintf(buf, "%0.1f MB", sz);
+	} else if (size >= 1000LL) {
+		sz = ((double)size / 1000.0f);
+		sprintf(buf, "%0.1f kB", sz);
+	} else {
+		sprintf(buf, "%d Bytes", (int)size);
+	}
+	return strdup(buf);
+}
+
 static plist_t mobilebackup_factory_info_plist_new()
 {
 	/* gather data from lockdown */
-	GTimeVal tv = {0, 0};
 	plist_t value_node = NULL;
 	plist_t root_node = NULL;
-	char *uuid = NULL;
-	char *uuid_uppercase = NULL;
+	char *udid = NULL;
+	char *udid_uppercase = NULL;
 
 	plist_t ret = plist_new_dict();
 
@@ -186,8 +280,7 @@ static plist_t mobilebackup_factory_info_plist_new()
 	if (value_node)
 		plist_dict_insert_item(ret, "IMEI", plist_copy(value_node));
 
-	g_get_current_time(&tv);
-	plist_dict_insert_item(ret, "Last Backup Date", plist_new_date(tv.tv_sec, 0));
+	plist_dict_insert_item(ret, "Last Backup Date", plist_new_date(time(NULL), 0));
 
 	value_node = plist_dict_get_item(root_node, "PhoneNumber");
 	if (value_node && (plist_get_node_type(value_node) == PLIST_STRING)) {
@@ -206,16 +299,16 @@ static plist_t mobilebackup_factory_info_plist_new()
 	/* FIXME Sync Settings? */
 
 	value_node = plist_dict_get_item(root_node, "UniqueDeviceID");
-	idevice_get_uuid(phone, &uuid);
-	plist_dict_insert_item(ret, "Target Identifier", plist_new_string(uuid));
+	idevice_get_udid(phone, &udid);
+	plist_dict_insert_item(ret, "Target Identifier", plist_new_string(udid));
 
 	plist_dict_insert_item(ret, "Target Type", plist_new_string("Device"));
 
 	/* uppercase */
-	uuid_uppercase = g_ascii_strup(uuid, -1);
-	plist_dict_insert_item(ret, "Unique Identifier", plist_new_string(uuid_uppercase));
-	free(uuid_uppercase);
-	free(uuid);
+	udid_uppercase = str_toupper(udid);
+	plist_dict_insert_item(ret, "Unique Identifier", plist_new_string(udid_uppercase));
+	free(udid_uppercase);
+	free(udid);
 
 	char *data_buf = NULL;
 	uint64_t data_size = 0;
@@ -243,9 +336,11 @@ static plist_t mobilebackup_factory_info_plist_new()
 	for (i = 0; itunesfiles[i]; i++) {
 		data_buf = NULL;
 		data_size = 0;
-		gchar *fname = g_strconcat("/iTunes_Control/iTunes/", itunesfiles[i], NULL);
+		char *fname = (char*)malloc(strlen("/iTunes_Control/iTunes/") + strlen(itunesfiles[i]) + 1);
+		strcpy(fname, "/iTunes_Control/iTunes/");
+		strcat(fname, itunesfiles[i]);
 		mobilebackup_afc_get_file_contents(fname, &data_buf, &data_size);
-		g_free(fname);
+		free(fname);
 		if (data_buf) {
 			plist_dict_insert_item(files, itunesfiles[i], plist_new_data(data_buf, data_size));
 			free(data_buf);
@@ -351,14 +446,14 @@ static int plist_write_to_filename(plist_t plist, const char *filename, enum pli
 	return 1;
 }
 
-static int mb2_status_check_snapshot_state(const char *path, const char *uuid, const char *matches)
+static int mb2_status_check_snapshot_state(const char *path, const char *udid, const char *matches)
 {
 	int ret = -1;
 	plist_t status_plist = NULL;
-	gchar *file_path = g_build_path(G_DIR_SEPARATOR_S, path, uuid, "Status.plist", NULL);
+	char *file_path = build_path(path, udid, "Status.plist", NULL);
 
 	plist_read_from_filename(&status_plist, file_path);
-	g_free(file_path);
+	free(file_path);
 	if (!status_plist) {
 		printf("Could not read Status.plist!\n");
 		return ret;
@@ -393,7 +488,7 @@ static int mobilebackup_info_is_current_device(plist_t info)
 	/* get basic device information in one go */
 	lockdownd_get_value(client, NULL, NULL, &root_node);
 
-	/* verify UUID */
+	/* verify UDID */
 	value_node = plist_dict_get_item(root_node, "UniqueDeviceID");
 	node = plist_dict_get_item(info, "Target Identifier");
 
@@ -483,7 +578,7 @@ static void print_progress_real(double progress, int flush)
 
 static void print_progress(uint64_t current, uint64_t total)
 {
-	gchar *format_size = NULL;
+	char *format_size = NULL;
 	double progress = ((double)current/(double)total)*100;
 	if (progress < 0)
 		return;
@@ -493,12 +588,12 @@ static void print_progress(uint64_t current, uint64_t total)
 
 	print_progress_real((double)progress, 0);
 
-	format_size = g_format_size_for_display(current);
+	format_size = format_size_for_display(current);
 	PRINT_VERBOSE(1, " (%s", format_size);
-	g_free(format_size);
-	format_size = g_format_size_for_display(total);
+	free(format_size);
+	format_size = format_size_for_display(total);
 	PRINT_VERBOSE(1, "/%s)     ", format_size);
-	g_free(format_size);
+	free(format_size);
 
 	fflush(stdout);
 	if (progress == 100)
@@ -531,7 +626,7 @@ static int mb2_handle_send_file(const char *backup_dir, const char *path, plist_
 	uint32_t nlen = 0;
 	uint32_t pathlen = strlen(path);
 	uint32_t bytes = 0;
-	gchar *localfile = g_build_path(G_DIR_SEPARATOR_S, backup_dir, path, NULL);
+	char *localfile = build_path(backup_dir, path, NULL);
 	char buf[32768];
 	struct stat fst;
 
@@ -546,7 +641,7 @@ static int mb2_handle_send_file(const char *backup_dir, const char *path, plist_
 	mobilebackup2_error_t err;
 
 	/* send path length */
-	nlen = GUINT32_TO_BE(pathlen);
+	nlen = htobe32(pathlen);
 	err = mobilebackup2_send_raw(mobilebackup2, (const char*)&nlen, sizeof(nlen), &bytes);
 	if (err != MOBILEBACKUP2_E_SUCCESS) {
 		goto leave_proto_err;
@@ -575,9 +670,9 @@ static int mb2_handle_send_file(const char *backup_dir, const char *path, plist_
 
 	total = fst.st_size;
 
-	gchar *format_size = g_format_size_for_display(total);
+	char *format_size = format_size_for_display(total);
 	PRINT_VERBOSE(1, "Sending '%s' (%s)\n", path, format_size);
-	g_free(format_size);
+	free(format_size);
 
 	if (total == 0) {
 		errcode = 0;
@@ -595,7 +690,7 @@ static int mb2_handle_send_file(const char *backup_dir, const char *path, plist_
 	do {
 		length = ((total-sent) < (off_t)sizeof(buf)) ? (uint32_t)total-sent : (uint32_t)sizeof(buf);
 		/* send data size (file size + 1) */
-		nlen = GUINT32_TO_BE(length+1);
+		nlen = htobe32(length+1);
 		memcpy(buf, &nlen, sizeof(nlen));
 		buf[4] = CODE_FILE_DATA;
 		err = mobilebackup2_send_raw(mobilebackup2, (const char*)buf, 5, &bytes);
@@ -631,7 +726,7 @@ leave:
 	if (errcode == 0) {
 		result = 0;
 		nlen = 1;
-		nlen = GUINT32_TO_BE(nlen);
+		nlen = htobe32(nlen);
 		memcpy(buf, &nlen, 4);
 		buf[4] = CODE_SUCCESS;
 		mobilebackup2_send_raw(mobilebackup2, buf, 5, &bytes);
@@ -643,7 +738,7 @@ leave:
 		mb2_multi_status_add_file_error(*errplist, path, errno_to_device_error(errcode), errdesc);
 		
 		length = strlen(errdesc);
-		nlen = GUINT32_TO_BE(length+1);
+		nlen = htobe32(length+1);
 		memcpy(buf, &nlen, 4);
 		buf[4] = CODE_ERROR_LOCAL;
 		slen = 5;
@@ -661,7 +756,7 @@ leave:
 leave_proto_err:
 	if (f)
 		fclose(f);
-	g_free(localfile);
+	free(localfile);
 	return result;
 }
 
@@ -689,10 +784,12 @@ static void mb2_handle_send_files(plist_t message, const char *backup_dir)
 			continue;
 
 		if (mb2_handle_send_file(backup_dir, str, &errplist) < 0) {
+			free(str);
 			//printf("Error when sending file '%s' to device\n", str);
 			// TODO: perhaps we can continue, we've got a multi status response?!
 			break;
 		}
+		free(str);
 	}
 
 	/* send terminating 0 dword */
@@ -700,7 +797,9 @@ static void mb2_handle_send_files(plist_t message, const char *backup_dir)
 	mobilebackup2_send_raw(mobilebackup2, (char*)&zero, 4, &sent);
 
 	if (!errplist) {
-		mobilebackup2_send_status_response(mobilebackup2, 0, NULL, plist_new_dict());
+		plist_t emptydict = plist_new_dict();
+		mobilebackup2_send_status_response(mobilebackup2, 0, NULL, emptydict);
+		plist_free(emptydict);
 	} else {
 		mobilebackup2_send_status_response(mobilebackup2, -13, "Multi status", errplist);
 		plist_free(errplist);
@@ -719,7 +818,7 @@ static int mb2_handle_receive_files(plist_t message, const char *backup_dir)
 	char buf[32768];
 	char *fname = NULL;
 	char *dname = NULL;
-	gchar *bname = NULL;
+	char *bname = NULL;
 	char code = 0;
 	char last_code = 0;
 	plist_t node = NULL;
@@ -741,7 +840,7 @@ static int mb2_handle_receive_files(plist_t message, const char *backup_dir)
 			break;
 		r = 0;
 		mobilebackup2_receive_raw(mobilebackup2, (char*)&nlen, 4, &r);
-		nlen = GUINT32_FROM_BE(nlen);
+		nlen = be32toh(nlen);
 		if (nlen == 0) {
 			// we're done here
 			break;
@@ -762,7 +861,7 @@ static int mb2_handle_receive_files(plist_t message, const char *backup_dir)
 		dname[r] = 0;
 		nlen = 0;
 		mobilebackup2_receive_raw(mobilebackup2, (char*)&nlen, 4, &r);
-		nlen = GUINT32_FROM_BE(nlen);
+		nlen = be32toh(nlen);
 		if (nlen == 0) {
 			printf("ERROR: %s: zero-length backup filename!\n", __func__);
 			break;
@@ -778,8 +877,8 @@ static int mb2_handle_receive_files(plist_t message, const char *backup_dir)
 		}
 		fname[r] = 0;
 		if (bname != NULL)
-			g_free(bname);
-		bname = g_build_path(G_DIR_SEPARATOR_S, backup_dir, fname, NULL);
+			free(bname);
+		bname = build_path(backup_dir, fname, NULL);
 		free(fname);
 		nlen = 0;
 		mobilebackup2_receive_raw(mobilebackup2, (char*)&nlen, 4, &r);
@@ -787,7 +886,7 @@ static int mb2_handle_receive_files(plist_t message, const char *backup_dir)
 			printf("ERROR: %s: could not receive code length!\n", __func__);
 			break;
 		}
-		nlen = GUINT32_FROM_BE(nlen);
+		nlen = be32toh(nlen);
 		last_code = code;
 		code = 0;
 		mobilebackup2_receive_raw(mobilebackup2, &code, 1, &r);
@@ -830,7 +929,7 @@ static int mb2_handle_receive_files(plist_t message, const char *backup_dir)
 				break;
 			nlen = 0;
 			mobilebackup2_receive_raw(mobilebackup2, (char*)&nlen, 4, &r);
-			nlen = GUINT32_FROM_BE(nlen);
+			nlen = be32toh(nlen);
 			if (nlen > 0) {
 				last_code = code;
 				mobilebackup2_receive_raw(mobilebackup2, &code, 1, &r);
@@ -873,7 +972,7 @@ static int mb2_handle_receive_files(plist_t message, const char *backup_dir)
 
 	/* clean up */
 	if (bname != NULL)
-		g_free(bname);
+		free(bname);
 
 	if (dname != NULL)
 		free(dname);
@@ -898,37 +997,40 @@ static void mb2_handle_list_directory(plist_t message, const char *backup_dir)
 		return;
 	}
 
-	gchar *path = g_build_path(G_DIR_SEPARATOR_S, backup_dir, str, NULL);
+	char *path = build_path(backup_dir, str, NULL);
 	free(str);
 
 	plist_t dirlist = plist_new_dict();
 
-	GDir *cur_dir = g_dir_open(path, 0, NULL);
+	DIR* cur_dir = opendir(path);
 	if (cur_dir) {
-		gchar *dir_file;
-		while ((dir_file = (gchar *)g_dir_read_name(cur_dir))) {
-			gchar *fpath = g_build_filename(path, dir_file, NULL);
+		struct dirent* ep;
+		while ((ep = readdir(cur_dir))) {
+			if ((strcmp(ep->d_name, ".") == 0) || (strcmp(ep->d_name, "..") == 0)) {
+				continue;
+			}
+			char *fpath = build_path(path, ep->d_name, NULL);
 			if (fpath) {
 				plist_t fdict = plist_new_dict();
-				GStatBuf st;
-				g_stat(fpath, &st);
+				struct stat st;
+				stat(fpath, &st);
 				const char *ftype = "DLFileTypeUnknown";
-				if (g_file_test(fpath, G_FILE_TEST_IS_DIR)) {
+				if (S_ISDIR(st.st_mode)) {
 					ftype = "DLFileTypeDirectory";
-				} else if (g_file_test(fpath, G_FILE_TEST_IS_REGULAR)) {
+				} else if (S_ISREG(st.st_mode)) {
 					ftype = "DLFileTypeRegular";
 				}
 				plist_dict_insert_item(fdict, "DLFileType", plist_new_string(ftype));
 				plist_dict_insert_item(fdict, "DLFileSize", plist_new_uint(st.st_size));
 				plist_dict_insert_item(fdict, "DLFileModificationDate", plist_new_date(st.st_mtime, 0));
 
-				plist_dict_insert_item(dirlist, dir_file, fdict);
-				g_free(fpath);
+				plist_dict_insert_item(dirlist, ep->d_name, fdict);
+				free(fpath);
 			}
 		}
-		g_dir_close(cur_dir);
+		closedir(cur_dir);
 	}
-	g_free(path);
+	free(path);
 
 	/* TODO error handling */
 	mobilebackup2_error_t err = mobilebackup2_send_status_response(mobilebackup2, 0, NULL, dirlist);
@@ -948,24 +1050,24 @@ static void mb2_handle_make_directory(plist_t message, const char *backup_dir)
 	char *errdesc = NULL;
 	plist_get_string_val(dir, &str);
 
-	gchar *newpath = g_build_path(G_DIR_SEPARATOR_S, backup_dir, str, NULL);
-	g_free(str);
+	char *newpath = build_path(backup_dir, str, NULL);
+	free(str);
 
-	if (g_mkdir_with_parents(newpath, 0755) < 0) {
+	if (mkdir_with_parents(newpath, 0755) < 0) {
 		errdesc = strerror(errno);
 		if (errno != EEXIST) {
 			printf("mkdir: %s (%d)\n", errdesc, errno);
 		}
 		errcode = errno_to_device_error(errno);
 	}
-	g_free(newpath);
+	free(newpath);
 	mobilebackup2_error_t err = mobilebackup2_send_status_response(mobilebackup2, errcode, errdesc, NULL);
 	if (err != MOBILEBACKUP2_E_SUCCESS) {
 		printf("Could not send status response, error %d\n", err);
 	}
 }
 
-static void mb2_copy_file_by_path(const gchar *src, const gchar *dst)
+static void mb2_copy_file_by_path(const char *src, const char *dst)
 {
 	FILE *from, *to;
 	char buf[BUFSIZ];
@@ -997,43 +1099,48 @@ static void mb2_copy_file_by_path(const gchar *src, const gchar *dst)
 	}
 }
 
-static void mb2_copy_directory_by_path(const gchar *src, const gchar *dst)
+static void mb2_copy_directory_by_path(const char *src, const char *dst)
 {
 	if (!src || !dst) {
 		return;
 	}
 
+	struct stat st;
+
 	/* if src does not exist */
-	if (!g_file_test(src, G_FILE_TEST_EXISTS)) {
+	if ((stat(src, &st) < 0) || !S_ISDIR(st.st_mode)) {
 		printf("ERROR: Source directory does not exist '%s': %s (%d)\n", src, strerror(errno), errno);
 		return;
 	}
 
 	/* if dst directory does not exist */
-	if (!g_file_test(dst, G_FILE_TEST_IS_DIR)) {
+	if ((stat(dst, &st) < 0) || !S_ISDIR(st.st_mode)) {
 		/* create it */
-		if (g_mkdir_with_parents(dst, 0755) < 0) {
+		if (mkdir_with_parents(dst, 0755) < 0) {
 			printf("ERROR: Unable to create destination directory '%s': %s (%d)\n", dst, strerror(errno), errno);
 			return;
 		}
 	}
 
 	/* loop over src directory contents */
-	GDir *cur_dir = g_dir_open(src, 0, NULL);
+	DIR *cur_dir = opendir(src);
 	if (cur_dir) {
-		gchar *dir_file;
-		while ((dir_file = (gchar *)g_dir_read_name(cur_dir))) {
-			gchar *srcpath = g_build_filename(src, dir_file, NULL);
-			gchar *dstpath = g_build_filename(dst, dir_file, NULL);
+		struct dirent* ep;
+		while ((ep = readdir(cur_dir))) {
+			if ((strcmp(ep->d_name, ".") == 0) || (strcmp(ep->d_name, "..") == 0)) {
+				continue;
+			}
+			char *srcpath = build_path(src, ep->d_name, NULL);
+			char *dstpath = build_path(dst, ep->d_name, NULL);
 			if (srcpath && dstpath) {
 				/* copy file */
 				mb2_copy_file_by_path(srcpath, dstpath);
 
-				g_free(srcpath);
-				g_free(dstpath);
+				free(srcpath);
+				free(dstpath);
 			}
 		}
-		g_dir_close(cur_dir);
+		closedir(cur_dir);
 	}
 }
 
@@ -1064,7 +1171,7 @@ static void print_usage(int argc, char **argv)
 	printf("  unback\tunpack a completed backup in DIRECTORY/_unback_/\n\n");
 	printf("options:\n");
 	printf("  -d, --debug\t\tenable communication debugging\n");
-	printf("  -u, --uuid UUID\ttarget specific device by its 40-digit device UUID\n");
+	printf("  -u, --udid UDID\ttarget specific device by its 40-digit device UDID\n");
 	printf("  -h, --help\t\tprints usage information\n");
 	printf("\n");
 }
@@ -1073,9 +1180,9 @@ int main(int argc, char *argv[])
 {
 	idevice_error_t ret = IDEVICE_E_UNKNOWN_ERROR;
 	int i;
-	char uuid[41];
+	char udid[41];
 	uint16_t port = 0;
-	uuid[0] = 0;
+	udid[0] = 0;
 	int cmd = -1;
 	int cmd_flags = 0;
 	int is_full_backup = 0;
@@ -1088,9 +1195,11 @@ int main(int argc, char *argv[])
 
 	/* we need to exit cleanly on running backups and restores or we cause havok */
 	signal(SIGINT, clean_exit);
-	signal(SIGQUIT, clean_exit);
 	signal(SIGTERM, clean_exit);
+#ifndef WIN32	
+	signal(SIGQUIT, clean_exit);
 	signal(SIGPIPE, SIG_IGN);
+#endif
 
 	/* parse cmdline args */
 	for (i = 1; i < argc; i++) {
@@ -1098,13 +1207,13 @@ int main(int argc, char *argv[])
 			idevice_set_debug_level(1);
 			continue;
 		}
-		else if (!strcmp(argv[i], "-u") || !strcmp(argv[i], "--uuid")) {
+		else if (!strcmp(argv[i], "-u") || !strcmp(argv[i], "--udid")) {
 			i++;
 			if (!argv[i] || (strlen(argv[i]) != 40)) {
 				print_usage(argc, argv);
 				return 0;
 			}
-			strcpy(uuid, argv[i]);
+			strcpy(udid, argv[i]);
 			continue;
 		}
 		else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
@@ -1168,10 +1277,10 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
-	if (uuid[0] != 0) {
-		ret = idevice_new(&phone, uuid);
+	if (udid[0] != 0) {
+		ret = idevice_new(&phone, udid);
 		if (ret != IDEVICE_E_SUCCESS) {
-			printf("No device found with uuid %s, is it plugged in?\n", uuid);
+			printf("No device found with udid %s, is it plugged in?\n", udid);
 			return -1;
 		}
 	}
@@ -1182,18 +1291,18 @@ int main(int argc, char *argv[])
 			printf("No device found, is it plugged in?\n");
 			return -1;
 		}
-		char *newuuid = NULL;
-		idevice_get_uuid(phone, &newuuid);
-		strcpy(uuid, newuuid);
-		free(newuuid);
+		char *newudid = NULL;
+		idevice_get_udid(phone, &newudid);
+		strcpy(udid, newudid);
+		free(newudid);
 	}
 
 	/* backup directory must contain an Info.plist */
-	gchar *info_path = g_build_path(G_DIR_SEPARATOR_S, backup_directory, uuid, "Info.plist", NULL);
+	char *info_path = build_path(backup_directory, udid, "Info.plist", NULL);
 	if (cmd == CMD_RESTORE) {
 		if (stat(info_path, &st) != 0) {
-			g_free(info_path);
-			printf("ERROR: Backup directory \"%s\" is invalid. No Info.plist found for UUID %s.\n", backup_directory, uuid);
+			free(info_path);
+			printf("ERROR: Backup directory \"%s\" is invalid. No Info.plist found for UDID %s.\n", backup_directory, udid);
 			return -1;
 		}
 	}
@@ -1321,9 +1430,9 @@ checkpoint:
 			PRINT_VERBOSE(1, "Starting backup...\n");
 
 			/* make sure backup device sub-directory exists */
-			gchar *devbackupdir = g_build_path(G_DIR_SEPARATOR_S, backup_directory, uuid, NULL);
-			g_mkdir(devbackupdir, 0755);
-			g_free(devbackupdir);
+			char *devbackupdir = build_path(backup_directory, udid, NULL);
+			__mkdir(devbackupdir, 0755);
+			free(devbackupdir);
 
 			/* TODO: check domain com.apple.mobile.backup key RequiresEncrypt and WillEncrypt with lockdown */
 			/* TODO: verify battery on AC enough battery remaining */	
@@ -1336,7 +1445,7 @@ checkpoint:
 			info_plist = mobilebackup_factory_info_plist_new();
 			remove(info_path);
 			plist_write_to_filename(info_plist, info_path, PLIST_FORMAT_XML);
-			g_free(info_path);
+			free(info_path);
 
 			plist_free(info_plist);
 			info_plist = NULL;
@@ -1344,7 +1453,7 @@ checkpoint:
 			/* request backup from device with manifest from last backup */
 			PRINT_VERBOSE(1, "Requesting backup from device...\n");
 
-			err = mobilebackup2_send_request(mobilebackup2, "Backup", uuid, NULL, NULL);
+			err = mobilebackup2_send_request(mobilebackup2, "Backup", udid, NULL, NULL);
 			if (err == MOBILEBACKUP2_E_SUCCESS) {
 				if (is_full_backup) {
 					PRINT_VERBOSE(1, "Full backup mode.\n");
@@ -1366,7 +1475,7 @@ checkpoint:
 			/* TODO: verify battery on AC enough battery remaining */
 
 			/* verify if Status.plist says we read from an successful backup */
-			if (!mb2_status_check_snapshot_state(backup_directory, uuid, "finished")) {
+			if (!mb2_status_check_snapshot_state(backup_directory, udid, "finished")) {
 				printf("ERROR: Cannot ensure we restore from a successful backup. Aborting.\n");
 				cmd = CMD_LEAVE;
 				break;
@@ -1386,7 +1495,7 @@ checkpoint:
 			plist_dict_insert_item(opts, "RestorePreserveSettings", plist_new_bool((cmd_flags & CMD_FLAG_RESTORE_SETTINGS) == 0));
 			PRINT_VERBOSE(1, "Preserve settings of device: %s\n", ((cmd_flags & CMD_FLAG_RESTORE_SETTINGS) == 0  ? "Yes":"No"));
 
-			err = mobilebackup2_send_request(mobilebackup2, "Restore", uuid, uuid, opts);
+			err = mobilebackup2_send_request(mobilebackup2, "Restore", udid, udid, opts);
 			plist_free(opts);
 			if (err != MOBILEBACKUP2_E_SUCCESS) {
 				if (err == MOBILEBACKUP2_E_BAD_VERSION) {
@@ -1401,7 +1510,7 @@ checkpoint:
 			break;
 			case CMD_INFO:
 			PRINT_VERBOSE(1, "Requesting backup info from device...\n");
-			err = mobilebackup2_send_request(mobilebackup2, "Info", uuid, NULL, NULL);
+			err = mobilebackup2_send_request(mobilebackup2, "Info", udid, NULL, NULL);
 			if (err != MOBILEBACKUP2_E_SUCCESS) {
 				printf("Error requesting backup info from device, error code %d\n", err);
 				cmd = CMD_LEAVE;
@@ -1409,7 +1518,7 @@ checkpoint:
 			break;
 			case CMD_LIST:
 			PRINT_VERBOSE(1, "Requesting backup list from device...\n");
-			err = mobilebackup2_send_request(mobilebackup2, "List", uuid, NULL, NULL);
+			err = mobilebackup2_send_request(mobilebackup2, "List", udid, NULL, NULL);
 			if (err != MOBILEBACKUP2_E_SUCCESS) {
 				printf("Error requesting backup list from device, error code %d\n", err);
 				cmd = CMD_LEAVE;
@@ -1417,7 +1526,7 @@ checkpoint:
 			break;
 			case CMD_UNBACK:
 			PRINT_VERBOSE(1, "Starting to unpack backup...\n");
-			err = mobilebackup2_send_request(mobilebackup2, "Unback", uuid, NULL, NULL);
+			err = mobilebackup2_send_request(mobilebackup2, "Unback", udid, NULL, NULL);
 			if (err != MOBILEBACKUP2_E_SUCCESS) {
 				printf("Error requesting unback operation from device, error code %d\n", err);
 				cmd = CMD_LEAVE;
@@ -1486,9 +1595,9 @@ checkpoint:
 								char *str = NULL;
 								plist_get_string_val(val, &str);
 								if (str) {
-									gchar *newpath = g_build_path(G_DIR_SEPARATOR_S, backup_directory, str, NULL);
-									g_free(str);
-									gchar *oldpath = g_build_path(G_DIR_SEPARATOR_S, backup_directory, key, NULL);
+									char *newpath = build_path(backup_directory, str, NULL);
+									free(str);
+									char *oldpath = build_path(backup_directory, key, NULL);
 
 									remove(newpath);
 									if (rename(oldpath, newpath) < 0) {
@@ -1497,8 +1606,8 @@ checkpoint:
 										errdesc = strerror(errno);
 										break;
 									}
-									g_free(oldpath);
-									g_free(newpath);
+									free(oldpath);
+									free(newpath);
 								}
 								free(key);
 								key = NULL;
@@ -1527,14 +1636,14 @@ checkpoint:
 							char *str = NULL;
 							plist_get_string_val(val, &str);
 							if (str) {
-								gchar *newpath = g_build_path(G_DIR_SEPARATOR_S, backup_directory, str, NULL);
-								g_free(str);
+								char *newpath = build_path(backup_directory, str, NULL);
+								free(str);
 								if (remove(newpath) < 0) {
 									printf("Could not remove '%s': %s (%d)\n", newpath, strerror(errno), errno);
 									errcode = errno_to_device_error(errno);
 									errdesc = strerror(errno);
 								}
-								g_free(newpath);
+								free(newpath);
 							}
 						}
 					}
@@ -1553,23 +1662,23 @@ checkpoint:
 						plist_get_string_val(srcpath, &src);
 						plist_get_string_val(dstpath, &dst);
 						if (src && dst) {
-							gchar *oldpath = g_build_path(G_DIR_SEPARATOR_S, backup_directory, src, NULL);
-							gchar *newpath = g_build_path(G_DIR_SEPARATOR_S, backup_directory, dst, NULL);
+							char *oldpath = build_path(backup_directory, src, NULL);
+							char *newpath = build_path(backup_directory, dst, NULL);
 
 							PRINT_VERBOSE(1, "Copying '%s' to '%s'\n", src, dst);
 
 							/* check that src exists */
-							if (g_file_test(oldpath, G_FILE_TEST_IS_DIR)) {
+							if ((stat(oldpath, &st) == 0) && S_ISDIR(st.st_mode)) {
 								mb2_copy_directory_by_path(oldpath, newpath);
-							} else if (g_file_test(oldpath, G_FILE_TEST_IS_REGULAR)) {
+							} else if ((stat(oldpath, &st) == 0) && S_ISREG(st.st_mode)) {
 								mb2_copy_file_by_path(oldpath, newpath);
 							}
 
-							g_free(newpath);
-							g_free(oldpath);
+							free(newpath);
+							free(oldpath);
 						}
-						g_free(src);
-						g_free(dst);
+						free(src);
+						free(dst);
 					}
 
 					err = mobilebackup2_send_status_response(mobilebackup2, errcode, errdesc, plist_new_dict());
@@ -1656,7 +1765,7 @@ files_out:
 			switch (cmd) {
 				case CMD_BACKUP:
 					PRINT_VERBOSE(1, "Received %d files from device.\n", file_count);
-					if (mb2_status_check_snapshot_state(backup_directory, uuid, "finished")) {
+					if (mb2_status_check_snapshot_state(backup_directory, udid, "finished")) {
 						PRINT_VERBOSE(1, "Backup Successful.\n");
 					} else {
 						if (quit_flag) {
