@@ -1,24 +1,22 @@
-/**
- * ideviceimagemounter -- Mount developer/debug disk images on the iPhone/iPod
+/*
+ * ideviceimagemounter.c
+ * Mount developer/debug disk images on the device
  *
  * Copyright (C) 2010 Nikias Bassen <nikias@gmx.li>
  *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more profile.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 
- * USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
  */
 
 #include <stdlib.h>
@@ -28,20 +26,22 @@
 #include <string.h>
 #include <getopt.h>
 #include <errno.h>
-#include <glib.h>
 #include <libgen.h>
+#include <time.h>
+#include <sys/time.h>
 
 #include <libimobiledevice/libimobiledevice.h>
 #include <libimobiledevice/lockdown.h>
 #include <libimobiledevice/afc.h>
 #include <libimobiledevice/notification_proxy.h>
 #include <libimobiledevice/mobile_image_mounter.h>
+#include <asprintf.h>
 
 static int indent_level = 0;
 
 static int list_mode = 0;
 static int xml_mode = 0;
-static char *uuid = NULL;
+static char *udid = NULL;
 static char *imagetype = NULL;
 
 static const char PKG_PATH[] = "PublicStaging";
@@ -54,7 +54,7 @@ static void print_usage(int argc, char **argv)
 	name = strrchr(argv[0], '/');
 	printf("Usage: %s [OPTIONS] IMAGE_FILE IMAGE_SIGNATURE_FILE\n\n", (name ? name + 1: argv[0]));
 	printf("Mounts the specified disk image on the device.\n\n");
-	printf("  -u, --uuid UUID\ttarget specific device by its 40-digit device UUID\n");
+	printf("  -u, --udid UDID\ttarget specific device by its 40-digit device UDID\n");
 	printf("  -l, --list\t\tList mount information\n");
 	printf("  -t, --imagetype\tImage type to use, default is 'Developer'\n");
 	printf("  -x, --xml\t\tUse XML output\n");
@@ -67,7 +67,7 @@ static void parse_opts(int argc, char **argv)
 {
 	static struct option longopts[] = {
 		{"help", 0, NULL, 'h'},
-		{"uuid", 0, NULL, 'u'},
+		{"udid", 0, NULL, 'u'},
 		{"list", 0, NULL, 'l'},
 		{"imagetype", 0, NULL, 't'},
 		{"xml", 0, NULL, 'x'},
@@ -89,12 +89,12 @@ static void parse_opts(int argc, char **argv)
 			exit(0);
 		case 'u':
 			if (strlen(optarg) != 40) {
-				printf("%s: invalid UUID specified (length != 40)\n",
+				printf("%s: invalid UDID specified (length != 40)\n",
 					   argv[0]);
 				print_usage(argc, argv);
 				exit(2);
 			}
-			uuid = strdup(optarg);
+			udid = strdup(optarg);
 			break;
 		case 'l':
 			list_mode = 1;
@@ -165,7 +165,7 @@ static void plist_node_to_string(plist_t node)
 	double d;
 	uint8_t b;
 	uint64_t u = 0;
-	GTimeVal tv = { 0, 0 };
+	struct timeval tv = { 0, 0 };
 
 	plist_type t;
 
@@ -210,14 +210,27 @@ static void plist_node_to_string(plist_t node)
 		}
 		free(data);
 		printf("\n");
-		g_free(s);
 		break;
 
 	case PLIST_DATE:
 		plist_get_date_val(node, (int32_t*)&tv.tv_sec, (int32_t*)&tv.tv_usec);
-		s = g_time_val_to_iso8601(&tv);
-		printf("%s\n", s);
-		free(s);
+		{
+			time_t ti = (time_t)tv.tv_sec;
+			struct tm *btime = localtime(&ti);
+			if (btime) {
+				s = (char*)malloc(24);
+ 				memset(s, 0, 24);
+				if (strftime(s, 24, "%Y-%m-%dT%H:%M:%SZ", btime) <= 0) {
+					free (s);
+					s = NULL;
+				}
+			}
+		}
+		if (s) {
+			puts(s);
+			free(s);
+		}
+		puts("\n");
 		break;
 
 	case PLIST_ARRAY:
@@ -280,7 +293,7 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (IDEVICE_E_SUCCESS != idevice_new(&device, uuid)) {
+	if (IDEVICE_E_SUCCESS != idevice_new(&device, udid)) {
 		printf("No device found, is it plugged in?\n");
 		return -1;
 	}
@@ -350,7 +363,7 @@ int main(int argc, char **argv)
 	} else {
 		char sig[8192];
 		size_t sig_length = 0;
-		FILE *f = fopen(image_sig_path, "r");
+		FILE *f = fopen(image_sig_path, "rb");
 		if (!f) {
 			fprintf(stderr, "Error opening signature file '%s': %s\n", image_sig_path, strerror(errno));
 			goto leave;
@@ -362,7 +375,7 @@ int main(int argc, char **argv)
 			goto leave;
 		}
 
-		f = fopen(image_path, "r");
+		f = fopen(image_path, "rb");
 		if (!f) {
 			fprintf(stderr, "Error opening image file '%s': %s\n", image_path, strerror(errno));
 			goto leave;
